'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = ArraySchema;

var _isPlainObj = require('is-plain-obj');

var _isPlainObj2 = _interopRequireDefault(_isPlainObj);

var _inherits = require('./utils/inherits');

var _inherits2 = _interopRequireDefault(_inherits);

var _object = require('./object');

var _object2 = _interopRequireDefault(_object);

var _mixed = require('./mixed');

var _mixed2 = _interopRequireDefault(_mixed);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ArraySchema() {
  if (!(this instanceof ArraySchema)) {
    return new ArraySchema();
  }

  _mixed2.default.call(this, { type: 'array' });
}

(0, _inherits2.default)(ArraySchema, _mixed2.default, {
  _getJSONSchema: function _getJSONSchema() {
    var opts = this._getOptions();

    var schema = {
      type: 'array'
    };

    if (opts.items != null) {
      if (Array.isArray(opts.items)) {
        (function () {
          var itemsSchema = [];

          opts.items.forEach(function (item) {
            var itemType = item;

            if ((0, _isPlainObj2.default)(item)) {
              itemType = new _object2.default().keys(item);
            }

            if (itemType instanceof _mixed2.default) {
              var generatedSchema = itemType.generate({ obj: true });

              if (Object.keys(generatedSchema).length > 0) {
                itemsSchema.push(generatedSchema);
              }
            }
          });

          if (itemsSchema.length > 0) {
            schema.items = itemsSchema;
          }
        })();
      } else if ((0, _isPlainObj2.default)(opts.items) || opts.items instanceof _mixed2.default) {
        var itemsType = opts.items,
            generatedSchema = void 0;

        if ((0, _isPlainObj2.default)(itemsType)) {
          itemsType = new _object2.default().keys(itemsType);
        }

        generatedSchema = itemsType.generate({ obj: true });

        if (Object.keys(generatedSchema).length > 0) {
          schema.items = generatedSchema;
        }
      }
    }

    if (opts.additionalItems != null) {
      schema.additionalItems = opts.additionalItems;
    }

    if (opts.minItems != null) {
      schema.minItems = opts.minItems;
    }

    if (opts.maxItems != null) {
      schema.maxItems = opts.maxItems;
    }

    if (opts.uniqueItems != null) {
      schema.uniqueItems = opts.uniqueItems;
    }

    return schema;
  },
  items: function items(value) {
    this._setOption('items', value);
    return this.instance();
  },
  additionalItems: function additionalItems(value) {
    this._setOption('additionalItems', value);
    return this.instance();
  },
  minItems: function minItems(value) {
    this._setOption('minItems', value);
    return this.instance();
  },
  maxItems: function maxItems(value) {
    this._setOption('maxItems', value);
    return this.instance();
  },
  uniqueItems: function uniqueItems(value) {
    this._setOption('uniqueItems', value);
    return this.instance();
  }
});
module.exports = exports['default'];