'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = SchemaType;

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _isPlainObj = require('is-plain-obj');

var _isPlainObj2 = _interopRequireDefault(_isPlainObj);

var _clone = require('./utils/clone');

var _clone2 = _interopRequireDefault(_clone);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function SchemaType() {
  var _this = this;

  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (!(this instanceof SchemaType)) {
    return new SchemaType();
  }

  this._options = {};

  this._type = options.type || 'mixed';

  if (this._type != null && this._type !== 'mixed') {
    this._options[this._type] = {};
  }

  if (this._type === 'mixed') {
    this.canBe = function () {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var types = void 0;

      if (Array.isArray(args[0])) {
        types = args[0];
      } else {
        types = args;
      }

      _this._setOption('types', types);

      return _this.instance();
    };

    this.ref = function (value) {
      _this._setOption('ref', value, true);
      return _this.instance();
    };
  }
} // @flow

SchemaType.prototype = {
  __isJsonSchemaSugar__: true,

  constructor: SchemaType,

  _getAllOptions: function _getAllOptions() {
    return this._options;
  },
  _getOptions: function _getOptions() {
    if (this._type != null && this._type !== 'mixed') {
      return this._options[this._type];
    }

    return this._options;
  },
  _setOption: function _setOption(optName, value, isRootOpt) {
    var localOpts = void 0;

    if (isRootOpt === true) {
      localOpts = this._getAllOptions();
    } else {
      localOpts = this._getOptions();
    }

    localOpts[optName] = value;
  },


  // return a new instance
  instance: function instance() {
    return this.clone();
  },


  // clone the shema
  clone: function clone() {
    return (0, _clone2.default)(this);
  },
  title: function title(value) {
    this._setOption('title', value, true);
    return this.instance();
  },
  description: function description(value) {
    this._setOption('description', value, true);
    return this.instance();
  },
  default: function _default(value) {
    this._setOption('default', value, true);
    return this.instance();
  },
  enum: function _enum(value) {
    this._setOption('enum', value, true);
    return this.instance();
  },
  required: function required() {
    this._setOption('required', true, true);
    return this.instance();
  },


  // property dependencies
  requiredWhen: function requiredWhen() {
    var presentProps = void 0;

    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    if (Array.isArray(args[0])) {
      presentProps = args[0];
    } else {
      presentProps = args;
    }

    this._setOption('requiredWhen', presentProps, true);
    return this.instance();
  },


  // schema dependencies
  when: function when(dependantProp, schema) {
    if (!(schema instanceof SchemaType)) {
      throw new Error('second parameter of `.when()` must be a schema');
    }

    this._setOption('when', {
      prop: dependantProp,
      schema: schema
    }, true);

    return this.instance();
  },


  // definitions of sub-schemas
  definitions: function definitions(value) {
    this._setOption('definitions', value, true);
    return this.instance();
  },
  allOf: function allOf(value) {
    this._setOption('allOf', value, true);
    return this.instance();
  },
  anyOf: function anyOf(value) {
    this._setOption('anyOf', value, true);
    return this.instance();
  },
  oneOf: function oneOf(value) {
    this._setOption('oneOf', value, true);
    return this.instance();
  },
  not: function not(value) {
    this._setOption('not', value, true);
    return this.instance();
  },
  noType: function noType() {
    this._setOption('noType', true, true);
    return this.instance();
  },
  generate: function generate() {
    var generateOptions = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var schema = this,
        rootOpts = this._getAllOptions(),
        getObj = generateOptions.obj,
        removeType = generateOptions.removeType,
        space = generateOptions.space,
        $schema = generateOptions.$schema,
        id = generateOptions.id;

    var jsonSchema = {},
        opts = void 0;

    if ($schema != null) {
      jsonSchema.$schema = $schema;
    }

    if (id != null) {
      jsonSchema.id = id;
    }

    if (this._type === 'mixed') {
      opts = this._getOptions();

      if (Array.isArray(opts.types)) {
        jsonSchema.type = [];

        opts.types.forEach(function (type) {
          if (type instanceof SchemaType) {
            var generatedSchema = type.generate({ obj: true });

            if (Object.keys(generatedSchema).length > 0) {
              jsonSchema.type.push(generatedSchema.type);
            }
          } else {
            jsonSchema.type.push(type);
          }
        });
      }

      if (opts.ref != null) {
        jsonSchema.$ref = opts.ref;
      }

      addMixedProps(jsonSchema, rootOpts);

      if (rootOpts.noType === true || removeType === true) {
        delete jsonSchema.type;
      }

      if (getObj === true) {
        return jsonSchema;
      }

      return JSON.stringify(jsonSchema, null, space);
    }

    if (typeof schema._getJSONSchema !== 'function') {
      if (getObj === true) {
        return {};
      }

      return '{}';
    }

    jsonSchema = (0, _objectAssign2.default)(jsonSchema, schema._getJSONSchema());

    addMixedProps(jsonSchema, rootOpts);

    if (rootOpts.noType === true || removeType === true) {
      delete jsonSchema.type;
    }

    if (getObj === true) {
      return jsonSchema;
    }

    return JSON.stringify(jsonSchema, null, space);
  }
};

function addMixedProps(jsonSchema, opts) {
  if (opts.title != null) {
    // eslint-disable-next-line no-param-reassign
    jsonSchema.title = opts.title;
  }

  if (opts.description != null) {
    // eslint-disable-next-line no-param-reassign
    jsonSchema.description = opts.description;
  }

  if (opts.default !== undefined) {
    // eslint-disable-next-line no-param-reassign
    jsonSchema.default = opts.default;
  }

  if (Array.isArray(opts.enum)) {
    // eslint-disable-next-line no-param-reassign
    jsonSchema.enum = opts.enum;
  }

  if (opts.definitions != null) {
    (function () {
      var definitionsSchema = opts.definitions,
          filteredDefinitions = {};

      if ((0, _isPlainObj2.default)(definitionsSchema)) {
        filteredDefinitions = Object.keys(definitionsSchema).reduce(function (obj, key) {
          var type = definitionsSchema[key];

          var generatedSchema = void 0;

          if (type instanceof SchemaType) {
            generatedSchema = type.generate({ obj: true });

            if (Object.keys(generatedSchema).length > 0) {
              // eslint-disable-next-line no-param-reassign
              obj[key] = generatedSchema;
            }
          }

          return obj;
        }, {});

        if (Object.keys(filteredDefinitions).length > 0) {
          // eslint-disable-next-line no-param-reassign
          jsonSchema.definitions = filteredDefinitions;
        }
      }
    })();
  }

  if (Array.isArray(opts.allOf)) {
    (function () {
      var allOfSchemas = [],
          generatedSchema = void 0;

      opts.allOf.forEach(function (type, index) {
        if (type instanceof SchemaType) {
          if (index === 0) {
            generatedSchema = type.generate({ obj: true });
          } else {
            generatedSchema = type.generate({ obj: true, removeType: true });
          }

          if (Object.keys(generatedSchema).length > 0) {
            allOfSchemas.push(generatedSchema);
          }
        }
      });

      if (allOfSchemas.length > 0) {
        // eslint-disable-next-line no-param-reassign
        jsonSchema.allOf = allOfSchemas;
      }
    })();
  }

  if (Array.isArray(opts.anyOf)) {
    (function () {
      var anyOfSchemas = [],
          generatedSchema = void 0;

      opts.anyOf.forEach(function (type) {
        if (type instanceof SchemaType) {
          generatedSchema = type.generate({ obj: true });

          if (Object.keys(generatedSchema).length > 0) {
            anyOfSchemas.push(generatedSchema);
          }
        }
      });

      if (anyOfSchemas.length > 0) {
        // eslint-disable-next-line no-param-reassign
        jsonSchema.anyOf = anyOfSchemas;
      }
    })();
  }

  if (Array.isArray(opts.oneOf)) {
    (function () {
      var oneOfSchemas = [],
          generatedSchema = void 0;

      opts.oneOf.forEach(function (type) {
        if (type instanceof SchemaType) {
          // when there is no type
          if (jsonSchema.type == null) {
            generatedSchema = type.generate({ obj: true });
          } else {
            generatedSchema = type.generate({ obj: true, removeType: true });
          }

          if (Object.keys(generatedSchema).length > 0) {
            oneOfSchemas.push(generatedSchema);
          }
        }
      });

      if (oneOfSchemas.length > 0) {
        // eslint-disable-next-line no-param-reassign
        jsonSchema.oneOf = oneOfSchemas;
      }
    })();
  }

  if (opts.not != null) {
    var _generatedSchema = void 0;

    if (opts.not instanceof SchemaType) {
      _generatedSchema = opts.not.generate({ obj: true });

      if (Object.keys(_generatedSchema).length > 0) {
        // eslint-disable-next-line no-param-reassign
        jsonSchema.not = _generatedSchema;
      }
    }
  }
}
module.exports = exports['default'];