'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = ObjectSchema;

var _lodash = require('lodash.omit');

var _lodash2 = _interopRequireDefault(_lodash);

var _inherits = require('./utils/inherits');

var _inherits2 = _interopRequireDefault(_inherits);

var _mixed = require('./mixed');

var _mixed2 = _interopRequireDefault(_mixed);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ObjectSchema() {
  if (!(this instanceof ObjectSchema)) {
    return new ObjectSchema();
  }

  _mixed2.default.call(this, { type: 'object' });
}

(0, _inherits2.default)(ObjectSchema, _mixed2.default, {
  _getJSONSchema: function _getJSONSchema() {
    var opts = this._getOptions();

    var requiredProperties = [],
        dependenciesProps = [],
        propDependenciesProps = [],
        dependencies = {},
        propDependencies = {},
        dependenciesKeys = void 0,
        propDependenciesKeys = void 0,
        filteredDependencies = {};

    var schema = {
      type: 'object'
    };

    if (opts.keys != null) {
      schema.properties = Object.keys(opts.keys).reduce(function (obj, key) {
        var valueType = opts.keys[key];

        var typeOpts = void 0,
            generatedSchema = void 0;

        // ignore values that are not schemas
        if (!(valueType instanceof _mixed2.default)) {
          return obj;
        }

        typeOpts = valueType._getAllOptions();

        if (typeOpts.required === true) {
          requiredProperties.push(key);
        }

        if (Array.isArray(typeOpts.requiredWhen)) {
          typeOpts.requiredWhen.forEach(function (prop) {
            if (propDependenciesProps.indexOf(prop) === -1 && prop !== key) {
              propDependenciesProps.push(prop);

              propDependencies[prop] = [];
            }

            // make sure that don't insert duplicate values
            if (Array.isArray(propDependencies[prop]) && propDependencies[prop].indexOf(key) === -1) {
              propDependencies[prop].push(key);
            }
          });
        }

        if (typeOpts.when != null) {
          var propDependant = typeOpts.when.prop,
              schemaExt = typeOpts.when.schema,
              schemaExtOpts = schemaExt._getAllOptions(),
              generatedSchemaExt = void 0;

          if (dependenciesProps.indexOf(propDependant) === -1 && propDependant !== key) {
            dependenciesProps.push(propDependant);

            dependencies[propDependant] = {
              properties: {}
            };
          }

          generatedSchemaExt = schemaExt.generate({ obj: true });

          if (Object.keys(generatedSchemaExt).length > 0) {
            dependencies[propDependant].properties[key] = generatedSchemaExt;
          }

          if (schemaExtOpts.required === true) {
            if (dependencies[propDependant].required == null) {
              dependencies[propDependant].required = [];
            }

            if (dependencies[propDependant].required.indexOf(key) === -1) {
              dependencies[propDependant].required.push(key);
            }
          }
        }

        generatedSchema = valueType.generate({ obj: true });

        // don't add empty schemas
        if (Object.keys(generatedSchema).length > 0) {
          // eslint-disable-next-line no-param-reassign
          obj[key] = generatedSchema;
        }

        return obj;
      }, {});

      if (requiredProperties.length > 0) {
        schema.required = requiredProperties;
      }

      dependenciesKeys = Object.keys(dependencies);
      propDependenciesKeys = Object.keys(propDependencies);

      if (dependenciesKeys.length > 0 && propDependenciesKeys.length > 0) {
        (function () {
          var filteredDep = void 0,
              filteredPropDep = void 0,
              restPropDep = void 0;

          filteredDep = filterDependencies(dependenciesKeys, dependencies);
          filteredPropDep = filterPropDependencies(propDependenciesKeys, propDependencies);

          restPropDep = (0, _lodash2.default)(filteredPropDep, Object.keys(filteredDep));

          filteredDependencies = Object.keys(filteredDep).reduce(function (obj, key) {
            if (filteredPropDep[key] != null) {
              if (!Array.isArray(filteredDep[key].required)) {
                filteredDep[key].required = filteredPropDep[key];
              } else {
                filteredPropDep[key].forEach(function (propDepValue) {
                  if (filteredDep[key].required.indexOf(propDepValue) === -1) {
                    filteredDep[key].required.push(propDepValue);
                  }
                });
              }
            }

            // eslint-disable-next-line no-param-reassign
            obj[key] = filteredDep[key];
            return obj;
          }, {});

          filteredDependencies = Object.keys(restPropDep).reduce(function (obj, key) {
            // eslint-disable-next-line no-param-reassign
            obj[key] = restPropDep[key];
            return obj;
          }, filteredDependencies);
        })();
      } else if (dependenciesKeys.length > 0) {
        filteredDependencies = filterDependencies(dependenciesKeys, dependencies);
      } else if (propDependenciesKeys.length > 0) {
        filteredDependencies = filterPropDependencies(propDependenciesKeys, propDependencies);
      }

      if (Object.keys(filteredDependencies).length > 0) {
        schema.dependencies = filteredDependencies;
      }
    }

    if (opts.additionalProperties != null) {
      if (opts.additionalProperties instanceof _mixed2.default) {
        var generatedSchema = opts.additionalProperties.generate({ obj: true });

        if (Object.keys(generatedSchema).length > 0) {
          schema.additionalProperties = generatedSchema;
        }
      } else {
        schema.additionalProperties = opts.additionalProperties;
      }
    }

    if (opts.patternProperties != null) {
      (function () {
        var patternProperties = {};

        Object.keys(opts.patternProperties).forEach(function (prop) {
          var propSchema = opts.patternProperties[prop];

          if (propSchema instanceof _mixed2.default) {
            var _generatedSchema = propSchema.generate({ obj: true });

            patternProperties[prop] = _generatedSchema;
          }
        });

        if (Object.keys(patternProperties).length > 0) {
          schema.patternProperties = patternProperties;
        }
      })();
    }

    if (opts.minProperties != null) {
      schema.minProperties = opts.minProperties;
    }

    if (opts.maxProperties != null) {
      schema.maxProperties = opts.maxProperties;
    }

    return schema;
  },
  keys: function keys(value) {
    this._setOption('keys', value);
    return this.instance();
  },
  additionalProperties: function additionalProperties(value) {
    this._setOption('additionalProperties', value);
    return this.instance();
  },
  patternProperties: function patternProperties(value) {
    this._setOption('patternProperties', value);
    return this.instance();
  },
  minProperties: function minProperties(value) {
    this._setOption('minProperties', value);
    return this.instance();
  },
  maxProperties: function maxProperties(value) {
    this._setOption('maxProperties', value);
    return this.instance();
  }
});

function filterDependencies(dependenciesKeys, dependencies) {
  return dependenciesKeys.reduce(function (obj, key) {
    if (Object.keys(dependencies[key].properties).length === 0) {
      return obj;
    }

    // eslint-disable-next-line no-param-reassign
    obj[key] = dependencies[key];
    return obj;
  }, {});
}

function filterPropDependencies(propDependenciesKeys, propDependencies) {
  return propDependenciesKeys.reduce(function (obj, key) {
    if (propDependencies[key].length === 0) {
      return obj;
    }

    // eslint-disable-next-line no-param-reassign
    obj[key] = propDependencies[key];
    return obj;
  }, {});
}
module.exports = exports['default'];